@@ -23,6 +23,7 @@ func (b *buffer) Free() {
 		bufPool.Put(b)
 	}
 }
+
 func (b *buffer) Write(bytes []byte) (int, error) {
 	*b = append(*b, bytes...)
 	return len(bytes), nil
@@ -37,10 +38,3 @@ func (b *buffer) WriteString(str string) (int, error) {
 	*b = append(*b, str...)
 	return len(str), nil
 }
-
-func (b *buffer) WriteStringIf(ok bool, str string) (int, error) {
-	if !ok {
-		return 0, nil
-	}
-	return b.WriteString(str)
-}
