@@ -10,36 +10,68 @@ import (
 	"slices"
 	"strconv"
 	"strings"
+	"sync"
 	"testing"
 	"time"
 
 	"github.com/lmittmann/tint"
 )
 
-var faketime = time.Date(2009, time.November, 10, 23, 0, 0, 0, time.UTC)
-
 func Example() {
-	slog.SetDefault(slog.New(tint.NewHandler(os.Stderr, &tint.Options{
+	w := os.Stderr
+	logger := slog.New(tint.NewHandler(w, &tint.Options{
 		Level:      slog.LevelDebug,
 		TimeFormat: time.Kitchen,
-	})))
-
-	slog.Info("Starting server", "addr", ":8080", "env", "production")
-	slog.Debug("Connected to DB", "db", "myapp", "host", "localhost:5432")
-	slog.Warn("Slow request", "method", "GET", "path", "/users", "duration", 497*time.Millisecond)
-	slog.Error("DB connection lost", tint.Err(errors.New("connection reset")), "db", "myapp")
+	}))
+	logger.Info("Starting server", "addr", ":8080", "env", "production")
+	logger.Debug("Connected to DB", "db", "myapp", "host", "localhost:5432")
+	logger.Warn("Slow request", "method", "GET", "path", "/users", "duration", 497*time.Millisecond)
+	logger.Error("DB connection lost", tint.Err(errors.New("connection reset")), "db", "myapp")
 	// Output:
 }
 
-// Run test with "faketime" tag:
-//
-//	TZ="" go test -tags=faketime
-func TestHandler(t *testing.T) {
-	if !faketime.Equal(time.Now()) {
-		t.Skip(`skipping test; run with "-tags=faketime"`)
-	}
+// Create a new logger that writes all errors in red:
+func Example_redErrors() {
+	w := os.Stderr
+	logger := slog.New(tint.NewHandler(w, &tint.Options{
+		ReplaceAttr: func(groups []string, a slog.Attr) slog.Attr {
+			if a.Value.Kind() == slog.KindAny {
+				if _, ok := a.Value.Any().(error); ok {
+					return tint.Attr(9, a)
+				}
+			}
+			return a
+		},
+	}))
+	logger.Error("DB connection lost", "error", errors.New("connection reset"), "db", "myapp")
+	// Output:
+}
 
-	tests := []struct {
+// Create a new logger with a custom TRACE level:
+func Example_traceLevel() {
+	const LevelTrace = slog.LevelDebug - 4
+
+	w := os.Stderr
+	logger := slog.New(tint.NewHandler(w, &tint.Options{
+		Level: LevelTrace,
+		ReplaceAttr: func(groups []string, a slog.Attr) slog.Attr {
+			if a.Key == slog.LevelKey && len(groups) == 0 {
+				level, ok := a.Value.Any().(slog.Level)
+				if ok && level <= LevelTrace {
+					return tint.Attr(13, slog.String(a.Key, "TRC"))
+				}
+			}
+			return a
+		},
+	}))
+	logger.Log(context.Background(), LevelTrace, "DB query", "query", "SELECT * FROM users", "duration", 543*time.Microsecond)
+	// Output:
+}
+
+var (
+	faketime = time.Date(2009, time.November, 10, 23, 0, 0, 0, time.UTC)
+
+	handlerTests = []struct {
 		Opts *tint.Options
 		F    func(l *slog.Logger)
 		Want string
@@ -100,7 +132,7 @@ func TestHandler(t *testing.T) {
 			F: func(l *slog.Logger) {
 				l.Info("test", "key", "val")
 			},
-			Want: `Nov 10 23:00:00.000 INF tint/handler_test.go:101 test key=val`,
+			Want: `Nov 10 23:00:00.000 INF tint/handler_test.go:133 test key=val`,
 		},
 		{
 			Opts: &tint.Options{
@@ -257,6 +289,168 @@ func TestHandler(t *testing.T) {
 			},
 			Want: `Nov 10 23:00:00.000 INF test ""=""`,
 		},
+		{
+			Opts: &tint.Options{
+				TimeFormat: time.DateOnly,
+				ReplaceAttr: func(groups []string, a slog.Attr) slog.Attr {
+					if len(groups) == 0 && a.Key == slog.TimeKey {
+						return slog.Time(slog.TimeKey, a.Value.Time().Add(24*time.Hour))
+					}
+					return a
+				},
+				NoColor: true,
+			},
+			F: func(l *slog.Logger) {
+				l.Info("test")
+			},
+			Want: `2009-11-11 INF test`,
+		},
+		{
+			F: func(l *slog.Logger) {
+				l.Info("test", "lvl", slog.LevelWarn)
+			},
+			Want: `Nov 10 23:00:00.000 INF test lvl=WARN`,
+		},
+		{
+			Opts: &tint.Options{NoColor: false},
+			F: func(l *slog.Logger) {
+				l.Info("test", "lvl", slog.LevelWarn)
+			},
+			Want: "\033[2mNov 10 23:00:00.000\033[0m \033[92mINF\033[0m test \033[2mlvl=\033[0mWARN",
+		},
+		{
+			Opts: &tint.Options{
+				ReplaceAttr: func(groups []string, a slog.Attr) slog.Attr {
+					return tint.Attr(13, a)
+				},
+			},
+			F: func(l *slog.Logger) {
+				l.Info("test")
+			},
+			Want: "\033[2;95mNov 10 23:00:00.000\033[0m \033[95mINF\033[0m \033[95mtest\033[0m",
+		},
+		{
+			Opts: &tint.Options{NoColor: false},
+			F: func(l *slog.Logger) {
+				l.Error("test", tint.Err(errors.New("fail")))
+			},
+			Want: "\033[2mNov 10 23:00:00.000\033[0m \033[91mERR\033[0m test \033[2;91merr=\033[22mfail\033[0m",
+		},
+		{
+			Opts: &tint.Options{NoColor: false},
+			F: func(l *slog.Logger) {
+				l.Info("test", tint.Attr(10, slog.String("key", "value")))
+			},
+			Want: "\033[2mNov 10 23:00:00.000\033[0m \033[92mINF\033[0m test \033[2;92mkey=\033[22mvalue\033[0m",
+		},
+		{
+			Opts: &tint.Options{NoColor: false},
+			F: func(l *slog.Logger) {
+				l.Info("test", tint.Attr(226, slog.String("key", "value")))
+			},
+			Want: "\033[2mNov 10 23:00:00.000\033[0m \033[92mINF\033[0m test \033[2;38;5;226mkey=\033[22mvalue\033[0m",
+		},
+		{
+			Opts: &tint.Options{
+				NoColor: false,
+				ReplaceAttr: func(groups []string, a slog.Attr) slog.Attr {
+					if a.Key == slog.MessageKey && len(groups) == 0 {
+						return tint.Attr(10, a)
+					}
+					return a
+				},
+			},
+			F: func(l *slog.Logger) {
+				l.Info("test", "key", "value")
+			},
+			Want: "\033[2mNov 10 23:00:00.000\033[0m \033[92mINF\033[0m \033[92mtest\033[0m \033[2mkey=\033[0mvalue",
+		},
+		{
+			Opts: &tint.Options{
+				NoColor: false,
+				ReplaceAttr: func(groups []string, a slog.Attr) slog.Attr {
+					if a.Key == slog.TimeKey && len(groups) == 0 {
+						return tint.Attr(10, a)
+					}
+					return a
+				},
+			},
+			F: func(l *slog.Logger) {
+				l.Info("test", "key", "value")
+			},
+			Want: "\033[2;92mNov 10 23:00:00.000\033[0m \033[92mINF\033[0m test \033[2mkey=\033[0mvalue",
+		},
+		{
+			Opts: &tint.Options{
+				NoColor: false,
+				ReplaceAttr: func(groups []string, a slog.Attr) slog.Attr {
+					if a.Key == slog.TimeKey && len(groups) == 0 {
+						return tint.Attr(10, slog.String(a.Key, a.Value.Time().Format(time.StampMilli)))
+					}
+					return a
+				},
+			},
+			F: func(l *slog.Logger) {
+				l.Info("test", "key", "value")
+			},
+			Want: "\033[2;92mNov 10 23:00:00.000\033[0m \033[92mINF\033[0m test \033[2mkey=\033[0mvalue",
+		},
+		{
+			Opts: &tint.Options{
+				AddSource: true,
+				NoColor:   false,
+				ReplaceAttr: func(groups []string, a slog.Attr) slog.Attr {
+					if a.Key == slog.SourceKey && len(groups) == 0 {
+						return tint.Attr(10, a)
+					}
+					return a
+				},
+			},
+			F: func(l *slog.Logger) {
+				l.Info("test")
+			},
+			Want: "\033[2mNov 10 23:00:00.000\033[0m \033[92mINF\033[0m \033[2;92mtint/handler_test.go:410\033[0m test",
+		},
+		{
+			Opts: &tint.Options{
+				NoColor: false,
+				Level:   slog.LevelDebug - 4,
+				ReplaceAttr: func(groups []string, a slog.Attr) slog.Attr {
+					if a.Key == slog.LevelKey && len(groups) == 0 {
+						level, ok := a.Value.Any().(slog.Level)
+						if ok && level <= slog.LevelDebug-4 {
+							return slog.String(a.Key, "TRC")
+						}
+					}
+					return a
+				},
+			},
+			F: func(l *slog.Logger) {
+				const levelTrace = slog.LevelDebug - 4
+				l.Log(context.TODO(), levelTrace, "test")
+			},
+			Want: "\033[2mNov 10 23:00:00.000\033[0m TRC test",
+		},
+		{
+			Opts: &tint.Options{
+				NoColor: false,
+				Level:   slog.LevelDebug - 4,
+				ReplaceAttr: func(groups []string, a slog.Attr) slog.Attr {
+					if a.Key == slog.LevelKey && len(groups) == 0 {
+						level, ok := a.Value.Any().(slog.Level)
+						if ok && level <= slog.LevelDebug-4 {
+							return tint.Attr(13, slog.String(a.Key, "TRC"))
+						}
+					}
+					return a
+				},
+			},
+			F: func(l *slog.Logger) {
+				const levelTrace = slog.LevelDebug - 4
+				l.Log(context.TODO(), levelTrace, "test")
+			},
+			Want: "\033[2mNov 10 23:00:00.000\033[0m \033[95mTRC\033[0m test",
+		},
 
 		{ // https://github.com/lmittmann/tint/issues/8
 			F: func(l *slog.Logger) {
@@ -345,7 +539,7 @@ func TestHandler(t *testing.T) {
 			F: func(l *slog.Logger) {
 				l.Info("test")
 			},
-			Want: `Nov 10 23:00:00.000 INF tint/handler_test.go:346 test`,
+			Want: `Nov 10 23:00:00.000 INF tint/handler_test.go:540 test`,
 		},
 		{ // https://github.com/lmittmann/tint/issues/44
 			F: func(l *slog.Logger) {
@@ -364,36 +558,28 @@ func TestHandler(t *testing.T) {
 			Want: `Nov 10 23:00:00.000 INF test key="{A:123 B:<nil>}"`,
 		},
 		{ // https://github.com/lmittmann/tint/issues/59
-			Opts: &tint.Options{
-				NoColor: false,
-			},
+			Opts: &tint.Options{NoColor: false},
 			F: func(l *slog.Logger) {
 				l.Info("test", "color", "\033[92mgreen\033[0m")
 			},
 			Want: "\033[2mNov 10 23:00:00.000\033[0m \033[92mINF\033[0m test \033[2mcolor=\033[0m\033[92mgreen\033[0m",
 		},
 		{
-			Opts: &tint.Options{
-				NoColor: false,
-			},
+			Opts: &tint.Options{NoColor: false},
 			F: func(l *slog.Logger) {
 				l.Info("test", "color", "\033[92mgreen quoted\033[0m")
 			},
 			Want: "\033[2mNov 10 23:00:00.000\033[0m \033[92mINF\033[0m test \033[2mcolor=\033[0m\"\033[92mgreen quoted\033[0m\"",
 		},
 		{
-			Opts: &tint.Options{
-				NoColor: true,
-			},
+			Opts: &tint.Options{NoColor: true},
 			F: func(l *slog.Logger) {
 				l.Info("test", "color", "\033[92mgreen\033[0m")
 			},
 			Want: `Nov 10 23:00:00.000 INF test color=green`,
 		},
 		{
-			Opts: &tint.Options{
-				NoColor: true,
-			},
+			Opts: &tint.Options{NoColor: true},
 			F: func(l *slog.Logger) {
 				l.Info("test", "color", "\033[92mgreen quoted\033[0m")
 			},
@@ -414,9 +600,45 @@ func TestHandler(t *testing.T) {
 			},
 			Want: `Nov 10 23:00:00.000 INF test time=<nil>`,
 		},
+		{ // https://github.com/lmittmann/tint/pull/94
+			F: func(l *slog.Logger) {
+				l.Info("test", "time", testTime)
+			},
+			Want: `Nov 10 23:00:00.000 INF test time=2022-05-01T00:00:00.000Z`,
+		},
+		{ // https://github.com/lmittmann/tint/pull/96
+			Opts: &tint.Options{
+				NoColor: false,
+				ReplaceAttr: func(groups []string, a slog.Attr) slog.Attr {
+					return a
+				},
+			},
+			F: func(l *slog.Logger) {
+				l.Info("test", tint.Attr(10, slog.String("key", "val")))
+			},
+			Want: "\033[2mNov 10 23:00:00.000\033[0m \033[92mINF\033[0m test \033[2;92mkey=\033[22mval\033[0m",
+		},
+		{
+			Opts: &tint.Options{
+				NoColor: false,
+				ReplaceAttr: func(groups []string, a slog.Attr) slog.Attr {
+					return tint.Attr(13, a)
+				},
+			},
+			F: func(l *slog.Logger) {
+				l.Info("test", tint.Attr(10, slog.String("key", "val")))
+			},
+			Want: "\033[2;95mNov 10 23:00:00.000\033[0m \033[95mINF\033[0m \033[95mtest\033[0m \033[2;95mkey=\033[22mval\033[0m",
+		},
+	}
+)
+
+func TestHandler(t *testing.T) {
+	if now := time.Now(); !faketime.Equal(now) || now.Location().String() != "UTC" {
+		t.Skip(`run: TZ="" go test -tags=faketime`)
 	}
 
-	for i, test := range tests {
+	for i, test := range handlerTests {
 		t.Run(strconv.Itoa(i), func(t *testing.T) {
 			var buf bytes.Buffer
 			if test.Opts == nil {
@@ -425,7 +647,10 @@ func TestHandler(t *testing.T) {
 			l := slog.New(tint.NewHandler(&buf, test.Opts))
 			test.F(l)
 
-			got := strings.TrimRight(buf.String(), "\n")
+			got, foundNewline := strings.CutSuffix(buf.String(), "\n")
+			if !foundNewline {
+				t.Fatalf("missing newline")
+			}
 			if test.Want != got {
 				t.Fatalf("(-want +got)\n- %s\n+ %s", test.Want, got)
 			}
@@ -465,8 +690,8 @@ func replace(new slog.Value, keys ...string) func([]string, slog.Attr) slog.Attr
 }
 
 func TestReplaceAttr(t *testing.T) {
-	if !faketime.Equal(time.Now()) {
-		t.Skip(`skipping test; run with "-tags=faketime"`)
+	if now := time.Now(); !faketime.Equal(now) || now.Location().String() != "UTC" {
+		t.Skip(`run: TZ="" go test -tags=faketime`)
 	}
 
 	tests := [][]any{
@@ -511,6 +736,54 @@ func TestReplaceAttr(t *testing.T) {
 	}
 }
 
+func TestAttr(t *testing.T) {
+	if now := time.Now(); !faketime.Equal(now) || now.Location().String() != "UTC" {
+		t.Skip(`run: TZ="" go test -tags=faketime`)
+	}
+
+	t.Run("text", func(t *testing.T) {
+		var buf bytes.Buffer
+		logger := slog.New(slog.NewTextHandler(&buf, nil))
+		logger.Info("test", tint.Attr(10, slog.String("key", "val")))
+
+		want := `time=2009-11-10T23:00:00.000Z level=INFO msg=test key=val`
+		if got := strings.TrimSpace(buf.String()); want != got {
+			t.Fatalf("(-want +got)\n- %s\n+ %s", want, got)
+		}
+	})
+
+	t.Run("json", func(t *testing.T) {
+		var buf bytes.Buffer
+		logger := slog.New(slog.NewJSONHandler(&buf, nil))
+		logger.Info("test", tint.Attr(10, slog.String("key", "val")))
+
+		want := `{"time":"2009-11-10T23:00:00Z","level":"INFO","msg":"test","key":"val"}`
+		if got := strings.TrimSpace(buf.String()); want != got {
+			t.Fatalf("(-want +got)\n- %s\n+ %s", want, got)
+		}
+	})
+}
+
+// TestClonedHandlersSynchronizeWriter tests that cloned handlers synchronize writer
+// writes with each other such that a logger can be shared among multiple goroutines.
+func TestClonedHandlersSynchronizeWriter(t *testing.T) {
+	// logSomething calls `With(...)` and uses the resulting logger to create and use a cloned handler.
+	logSomething := func(wg *sync.WaitGroup, logger *slog.Logger, loggerID int) {
+		defer wg.Done()
+		logger = logger.With("withLoggerID", loggerID)
+		logger.Info("test")
+	}
+
+	logger := slog.New(tint.NewHandler(&bytes.Buffer{}, &tint.Options{}))
+
+	// start and wait for two goroutines
+	var wg sync.WaitGroup
+	wg.Add(2)
+	go logSomething(&wg, logger, 1)
+	go logSomething(&wg, logger, 2)
+	wg.Wait()
+}
+
 // See https://github.com/golang/exp/blob/master/slog/benchmarks/benchmarks_test.go#L25
 //
 // Run e.g.:
@@ -627,6 +900,14 @@ func BenchmarkLogAttrs(b *testing.B) {
 				)
 			},
 		},
+		{
+			"attr",
+			func(logger *slog.Logger) {
+				logger.LogAttrs(context.TODO(), slog.LevelError, testMessage,
+					tint.Attr(9, slog.String("string", testString)),
+				)
+			},
+		},
 	}
 
 	for _, h := range handler {
